LIBRARY libsvm
EXPORTS
	svm_train	@1
	svm_cross_validation	@2
	svm_save_model	@3
	svm_load_model	@4
	svm_get_svm_type	@5
	svm_get_nr_class	@6
	svm_get_labels	@7
	svm_get_svr_probability	@8
	svm_predict_values	@9
	svm_predict	@10
	svm_predict_probability	@11
	svm_free_model_content	@12
	svm_free_and_destroy_model	@13
	svm_destroy_param	@14
	svm_check_parameter	@15
	svm_check_probability_model	@16
	svm_set_print_string_function	@17
	svm_get_sv_indices	@18
	svm_get_nr_sv	@19
